@@ -1169,7 +1169,7 @@ bool ParseHost(const std::string& input,
                std::string* output,
                bool is_special,
                bool unicode = false) {
-  if (input.length() == 0) {
+  if (input.empty()) {
     output->clear();
     return true;
   }
@@ -1487,7 +1487,7 @@ void URL::Parse(const char* input,
             state = kSpecialRelativeOrAuthority;
           } else if (special) {
             state = kSpecialAuthoritySlashes;
-          } else if (p[1] == '/') {
+          } else if (p + 1 < end && p[1] == '/') {
             state = kPathOrAuthority;
             p++;
           } else {
@@ -1547,7 +1547,7 @@ void URL::Parse(const char* input,
         }
         break;
       case kSpecialRelativeOrAuthority:
-        if (ch == '/' && p[1] == '/') {
+        if (ch == '/' && p + 1 < end && p[1] == '/') {
           state = kSpecialAuthorityIgnoreSlashes;
           p++;
         } else {
@@ -1695,7 +1695,7 @@ void URL::Parse(const char* input,
         break;
       case kSpecialAuthoritySlashes:
         state = kSpecialAuthorityIgnoreSlashes;
-        if (ch == '/' && p[1] == '/') {
+        if (ch == '/' && p + 1 < end && p[1] == '/') {
           p++;
         } else {
           continue;
@@ -2036,7 +2036,7 @@ void URL::Parse(const char* input,
               (ch == kEOL ||
                ch == '?' ||
                ch == '#')) {
-            while (url->path.size() > 1 && url->path[0].length() == 0) {
+            while (url->path.size() > 1 && url->path[0].empty()) {
               url->path.erase(url->path.begin());
             }
           }
@@ -2059,9 +2059,9 @@ void URL::Parse(const char* input,
             state = kFragment;
             break;
           default:
-            if (url->path.size() == 0)
+            if (url->path.empty())
               url->path.emplace_back("");
-            if (url->path.size() > 0 && ch != kEOL)
+            else if (ch != kEOL)
               AppendOrEscape(&url->path[0], ch, C0_CONTROL_ENCODE_SET);
         }
         break;
